@@ -1,6 +1,6 @@
 #!/bin/sh
 set -ev
-VERSION=4.53
+VERSION=5.18
 DST=stunnel-$VERSION-android
 
 # to build Zlib:
@@ -15,14 +15,19 @@ DST=stunnel-$VERSION-android
 # make
 # make install
 
-./configure --build=i686-pc-linux-gnu --host=arm-linux-androideabi --prefix=/data/local --with-ssl=/opt/androideabi/sysroot
+test -f Makefile && make distclean
+mkdir -p bin/android
+cd bin/android
+../../configure --with-sysroot --build=i686-pc-linux-gnu --host=arm-linux-androideabi --prefix=/data/local
 make clean
 make
+cd ../..
 mkdir $DST
-cp src/stunnel /opt/androideabi/sysroot/bin/openssl $DST
+cp bin/android/src/stunnel $DST
 # arm-linux-androideabi-strip $DST/stunnel $DST/openssl
-arm-linux-androideabi-strip $DST/openssl
+# cp /opt/androideabi/sysroot/bin/openssl $DST
+# arm-linux-androideabi-strip $DST/openssl
 zip -r $DST.zip $DST
 rm -rf $DST
-sha256sum $DST.zip
-mv $DST.zip ../dist/
+# sha256sum $DST.zip
+# mv $DST.zip ../dist/
