@@ -1,6 +1,6 @@
 /*
- *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   stunnel       TLS offloading and load-balancing proxy
+ *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -38,11 +38,14 @@
 #include "common.h"
 #include "prototypes.h"
 
-NOEXPORT void log_raw(const int, const char *, const char *, const char *);
+NOEXPORT void log_raw(const SERVICE_OPTIONS *, const int,
+    const char *, const char *, const char *);
+NOEXPORT void safestring(char *);
 
 static DISK_FILE *outfile=NULL;
 static struct LIST { /* single-linked list of log lines */
     struct LIST *next;
+    SERVICE_OPTIONS *opt;
     int level;
     char *stamp, *id, *text;
 } *head=NULL, *tail=NULL;
@@ -56,9 +59,10 @@ void syslog_open(void) {
     syslog_close();
     if(global_options.option.syslog)
 #ifdef __ultrix__
-        openlog("stunnel", 0);
+        openlog(service_options.servname, 0);
 #else
-        openlog("stunnel", LOG_CONS|LOG_NDELAY, global_options.facility);
+        openlog(service_options.servname,
+            LOG_CONS|LOG_NDELAY, global_options.log_facility);
 #endif /* __ultrix__ */
     syslog_opened=1;
 }
@@ -118,7 +122,7 @@ void log_flush(LOG_MODE new_mode) {
 
     enter_critical_section(CRIT_LOG);
     while(head) {
-        log_raw(head->level, head->stamp, head->id, head->text);
+        log_raw(head->opt, head->level, head->stamp, head->id, head->text);
         str_free(head->stamp);
         str_free(head->id);
         str_free(head->text);
@@ -133,17 +137,29 @@ void log_flush(LOG_MODE new_mode) {
 void s_log(int level, const char *format, ...) {
     va_list ap;
     char *text, *stamp, *id;
-    unsigned long tid;
     struct LIST *tmp;
-    int libc_error, socket_error;
+#ifdef USE_WIN32
+    DWORD libc_error;
+#else
+    int libc_error;
+#endif
+    int socket_error;
     time_t gmt;
     struct tm *timeptr;
 #if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
     struct tm timestruct;
 #endif
+    TLS_DATA *tls_data;
+
+    tls_data=tls_get();
+    if(!tls_data) {
+        tls_data=tls_alloc(NULL, NULL, "log");
+        s_log(LOG_ERR, "INTERNAL ERROR: Uninitialized TLS at %s, line %d",
+            __FILE__, __LINE__);
+    }
 
     /* performance optimization: skip the trivial case early */
-    if(mode==LOG_MODE_CONFIGURED && level>global_options.debug_level)
+    if(mode==LOG_MODE_CONFIGURED && level>tls_data->opt->log_level)
         return;
 
     libc_error=get_last_error();
@@ -159,10 +175,7 @@ void s_log(int level, const char *format, ...) {
     stamp=str_printf("%04d.%02d.%02d %02d:%02d:%02d",
         timeptr->tm_year+1900, timeptr->tm_mon+1, timeptr->tm_mday,
         timeptr->tm_hour, timeptr->tm_min, timeptr->tm_sec);
-    tid=stunnel_thread_id();
-    if(!tid) /* currently USE_FORK */
-        tid=stunnel_process_id();
-    id=str_printf("LOG%d[%lu]", level, tid);
+    id=str_printf("LOG%d[%s]", level, tls_data->id);
 
     /* format the text to be logged */
     va_start(ap, format);
@@ -172,9 +185,9 @@ void s_log(int level, const char *format, ...) {
 
     if(mode==LOG_MODE_NONE) { /* save the text to log it later */
         enter_critical_section(CRIT_LOG);
-        tmp=str_alloc(sizeof(struct LIST));
-        str_detach(tmp);
+        tmp=str_alloc_detached(sizeof(struct LIST));
         tmp->next=NULL;
+        tmp->opt=tls_data->opt;
         tmp->level=level;
         tmp->stamp=stamp;
         str_detach(tmp->stamp);
@@ -189,7 +202,7 @@ void s_log(int level, const char *format, ...) {
         tail=tmp;
         leave_critical_section(CRIT_LOG);
     } else { /* ready log the text directly */
-        log_raw(level, stamp, id, text);
+        log_raw(tls_data->opt, level, stamp, id, text);
         str_free(stamp);
         str_free(id);
         str_free(text);
@@ -199,14 +212,15 @@ void s_log(int level, const char *format, ...) {
     set_last_socket_error(socket_error);
 }
 
-NOEXPORT void log_raw(const int level, const char *stamp,
+NOEXPORT void log_raw(const SERVICE_OPTIONS *opt,
+        const int level, const char *stamp,
         const char *id, const char *text) {
     char *line;
 
     /* build the line and log it to syslog/file */
     if(mode==LOG_MODE_CONFIGURED) { /* configured */
         line=str_printf("%s %s: %s", stamp, id, text);
-        if(level<=global_options.debug_level) {
+        if(level<=opt->log_level) {
 #if !defined(USE_WIN32) && !defined(__vms)
             if(global_options.option.syslog)
                 syslog(level, "%s: %s", id, text);
@@ -226,9 +240,9 @@ NOEXPORT void log_raw(const int level, const char *stamp,
     if(mode==LOG_MODE_ERROR ||
             (mode==LOG_MODE_INFO && level<LOG_DEBUG) ||
 #if defined(USE_WIN32) || defined(USE_JNI)
-            level<=global_options.debug_level
+            level<=opt->log_level
 #else
-            (level<=global_options.debug_level &&
+            (level<=opt->log_level &&
             global_options.option.foreground)
 #endif
             )
@@ -237,12 +251,62 @@ NOEXPORT void log_raw(const int level, const char *stamp,
     str_free(line);
 }
 
-/* critical problem - str.c functions are not safe to use */
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat"
+#pragma GCC diagnostic ignored "-Wformat-extra-args"
+#endif /* __GNUC__ */
+char *log_id(CLI *c) {
+    static volatile unsigned long long seq=0;
+    unsigned long long my_seq;
+    const char table[62]=
+        "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    unsigned char rnd[22];
+    char *uniq;
+    size_t i;
+    unsigned long tid;
+
+    switch(c->opt->log_id) {
+    case LOG_ID_SEQENTIAL:
+        enter_critical_section(CRIT_ID);
+        my_seq=seq++;
+        leave_critical_section(CRIT_ID);
+        return str_printf("%llu", my_seq);
+    case LOG_ID_UNIQUE:
+        if(RAND_bytes(rnd, sizeof rnd)<=0) /* log2(62^22)=130.99 */
+            return str_dup("error");
+        for(i=0; i<sizeof rnd; ++i) {
+            rnd[i]&=63;
+            while(rnd[i]>=62) {
+                if(RAND_bytes(rnd+i, 1)<=0)
+                    return str_dup("error");
+                rnd[i]&=63;
+            }
+        }
+        uniq=str_alloc(sizeof rnd+1);
+        for(i=0; i<sizeof rnd; ++i)
+            uniq[i]=table[rnd[i]];
+        uniq[sizeof rnd]='\0';
+        return uniq;
+    case LOG_ID_THREAD:
+        tid=stunnel_thread_id();
+        if(!tid) /* currently USE_FORK */
+            tid=stunnel_process_id();
+        return str_printf("%lu", tid);
+    }
+    return str_dup("error");
+}
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif /* __GNUC__ */
+
+/* critical problem handling */
+/* str.c functions are not safe to use here */
 #ifdef __GNUC__
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wunused-result"
 #endif /* __GNUC__ */
-void fatal_debug(char *txt, char *file, int line) {
+void fatal_debug(char *txt, const char *file, int line) {
     char msg[80];
 #ifdef USE_WIN32
     DWORD num;
@@ -295,7 +359,7 @@ void fatal_debug(char *txt, char *file, int line) {
 #endif /* __GNUC__ */
 
 void ioerror(const char *txt) { /* input/output error */
-    log_error(LOG_ERR, get_last_error(), txt);
+    log_error(LOG_ERR, (int)get_last_error(), txt);
 }
 
 void sockerror(const char *txt) { /* socket error */
@@ -419,4 +483,11 @@ char *s_strerror(int errnum) {
     }
 }
 
+/* replace non-UTF-8 and non-printable control characters with '.' */
+NOEXPORT void safestring(char *c) {
+    for(; *c; ++c)
+        if(!(*c&0x80 || isprint(*c)))
+            *c='.';
+}
+
 /* end of log.c */
