@@ -1,6 +1,6 @@
 /*
- *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   stunnel       TLS offloading and load-balancing proxy
+ *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -39,25 +39,48 @@
 #include "prototypes.h"
 
     /* global OpenSSL initalization: compression, engine, entropy */
-NOEXPORT int init_compression(GLOBAL_OPTIONS *);
-NOEXPORT int init_prng(GLOBAL_OPTIONS *);
+NOEXPORT void cb_free(void *parent, void *ptr, CRYPTO_EX_DATA *ad,
+    int idx, long argl, void *argp);
+#ifndef OPENSSL_NO_COMP
+NOEXPORT int compression_init(GLOBAL_OPTIONS *);
+#endif
+NOEXPORT int prng_init(GLOBAL_OPTIONS *);
 NOEXPORT int add_rand_file(GLOBAL_OPTIONS *, const char *);
 
-int cli_index, opt_index; /* to keep structure for callbacks */
+int index_cli, index_opt, index_redirect, index_addr;
 
 int ssl_init(void) { /* init SSL before parsing configuration file */
     SSL_load_error_strings();
     SSL_library_init();
-    cli_index=SSL_get_ex_new_index(0, "cli index", NULL, NULL, NULL);
-    opt_index=SSL_CTX_get_ex_new_index(0, "opt index", NULL, NULL, NULL);
-    if(cli_index<0 || opt_index<0)
+    index_cli=SSL_get_ex_new_index(0, "cli index",
+        NULL, NULL, NULL);
+    index_opt=SSL_CTX_get_ex_new_index(0, "opt index",
+        NULL, NULL, NULL);
+    index_redirect=SSL_SESSION_get_ex_new_index(0, "redirect index",
+        NULL, NULL, NULL);
+    index_addr=SSL_SESSION_get_ex_new_index(0, "addr index",
+        NULL, NULL, cb_free);
+    if(index_cli<0 || index_opt<0 || index_redirect<0 || index_addr<0) {
+        s_log(LOG_ERR, "Application specific data initialization failed");
         return 1;
-#ifdef HAVE_OSSL_ENGINE_H
+    }
+#ifndef OPENSSL_NO_ENGINE
     ENGINE_load_builtin_engines();
 #endif
     return 0;
 }
 
+NOEXPORT void cb_free(void *parent, void *ptr, CRYPTO_EX_DATA *ad,
+        int idx, long argl, void *argp) {
+    (void)parent; /* skip warning about unused parameter */
+    (void)ad; /* skip warning about unused parameter */
+    (void)idx; /* skip warning about unused parameter */
+    (void)argl; /* skip warning about unused parameter */
+    s_log(LOG_DEBUG, "Deallocating application specific data for %s",
+        (char *)argp);
+    str_free(ptr);
+}
+
 int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global SSL settings */
 #ifdef USE_FIPS
     if(FIPS_mode()!=global->option.fips) {
@@ -71,21 +94,22 @@ int ssl_configure(GLOBAL_OPTIONS *global) { /* configure global SSL settings */
     s_log(LOG_NOTICE, "FIPS mode %s",
         global->option.fips ? "enabled" : "disabled");
 #endif /* USE_FIPS */
-    if(init_compression(global))
+#ifndef OPENSSL_NO_COMP
+    if(compression_init(global))
         return 1;
-    if(init_prng(global))
+#endif /* OPENSSL_NO_COMP */
+    if(prng_init(global))
         return 1;
     s_log(LOG_DEBUG, "PRNG seeded successfully");
     return 0; /* SUCCESS */
 }
 
-NOEXPORT int init_compression(GLOBAL_OPTIONS *global) {
 #ifndef OPENSSL_NO_COMP
-    SSL_COMP *comp;
-    STACK_OF(SSL_COMP) *ssl_comp_methods;
+NOEXPORT int compression_init(GLOBAL_OPTIONS *global) {
+    STACK_OF(SSL_COMP) *methods;
 
-    ssl_comp_methods=SSL_COMP_get_compression_methods();
-    if(!ssl_comp_methods) {
+    methods=SSL_COMP_get_compression_methods();
+    if(!methods) {
         if(global->compression==COMP_NONE) {
             s_log(LOG_NOTICE, "Failed to get compression methods");
             return 0; /* ignore */
@@ -95,68 +119,41 @@ NOEXPORT int init_compression(GLOBAL_OPTIONS *global) {
         }
     }
 
-    /* delete OpenSSL defaults (empty the SSL_COMP stack) */
-    /* cannot use sk_SSL_COMP_pop_free, as it also destroys the stack itself */
-    while(sk_SSL_COMP_num(ssl_comp_methods))
-        OPENSSL_free(sk_SSL_COMP_pop(ssl_comp_methods));
+    if(global->compression==COMP_NONE ||
+            SSLeay()<0x00908051L /* 0.9.8e-beta1 */) {
+        /* delete OpenSSL defaults (empty the SSL_COMP stack) */
+        /* cannot use sk_SSL_COMP_pop_free,
+         * as it also destroys the stack itself */
+        /* only leave the standard RFC 1951 (DEFLATE) algorithm,
+         * if any of the private algorithms is enabled */
+        /* only allow DEFLATE with OpenSSL 0.9.8 or later
+         * with OpenSSL #1468 zlib memory leak fixed */
+        while(sk_SSL_COMP_num(methods))
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
+            /* FIXME: remove when sk_SSL_COMP_pop() works again */
+            OPENSSL_free(sk_pop((void *)methods));
+#else
+            OPENSSL_free(sk_SSL_COMP_pop(methods));
+#endif
+    }
 
     if(global->compression==COMP_NONE) {
         s_log(LOG_DEBUG, "Compression disabled");
         return 0; /* success */
     }
 
-    /* insert RFC 1951 (DEFLATE) algorithm */
-    if(SSLeay()>=0x00908051L) { /* 0.9.8e-beta1 */
-        /* only allow DEFLATE with OpenSSL 0.9.8 or later
-           with openssl #1468 zlib memory leak fixed */
-        comp=(SSL_COMP *)OPENSSL_malloc(sizeof(SSL_COMP));
-        if(!comp) {
-            s_log(LOG_ERR, "OPENSSL_malloc filed");
-            return 1;
-        }
-        comp->id=1; /* RFC 1951 */
-        comp->method=COMP_zlib();
-        if(!comp->method || comp->method->type==NID_undef) {
-            OPENSSL_free(comp);
-            s_log(LOG_ERR, "Failed to initialize compression method");
-            return 1;
-        }
-        comp->name=(char *)(comp->method->name);
-        sk_SSL_COMP_push(ssl_comp_methods, comp);
-    }
-
-    /* also insert one of obsolete (ZLIB/RLE) algorithms */
-    comp=(SSL_COMP *)OPENSSL_malloc(sizeof(SSL_COMP));
-    if(!comp) {
-        s_log(LOG_ERR, "OPENSSL_malloc filed");
-        return 1;
-    }
+    /* also insert the obsolete ZLIB algorithm */
     if(global->compression==COMP_ZLIB) {
-        comp->id=0xe0; /* 224 - within private range (193 to 255) */
-        comp->method=COMP_zlib();
-    } else if(global->compression==COMP_RLE) {
-        comp->id=0xe1; /* 225 - within private range (193 to 255) */
-        comp->method=COMP_rle();
-    } else {
-        s_log(LOG_INFO, "Compression enabled: %d algorithm(s)",
-            sk_SSL_COMP_num(ssl_comp_methods));
-        OPENSSL_free(comp);
-        return 0;
+        /* 224 - within the private range (193 to 255) */
+        SSL_COMP_add_compression_method(0xe0, COMP_zlib());
     }
-    if(!comp->method || comp->method->type==NID_undef) {
-        OPENSSL_free(comp);
-        s_log(LOG_ERR, "Failed to initialize compression method");
-        return 1;
-    }
-    comp->name=(char *)(comp->method->name);
-    sk_SSL_COMP_push(ssl_comp_methods, comp);
-    s_log(LOG_INFO, "Compression enabled: %d algorithm(s)",
-        sk_SSL_COMP_num(ssl_comp_methods));
-#endif /* OPENSSL_NO_COMP */
+    s_log(LOG_INFO, "Compression enabled: %d method(s)",
+        sk_SSL_COMP_num(methods));
     return 0; /* success */
 }
+#endif /* OPENSSL_NO_COMP */
 
-NOEXPORT int init_prng(GLOBAL_OPTIONS *global) {
+NOEXPORT int prng_init(GLOBAL_OPTIONS *global) {
     int totbytes=0;
     char filename[256];
 #ifndef USE_WIN32
@@ -233,7 +230,7 @@ NOEXPORT int add_rand_file(GLOBAL_OPTIONS *global, const char *filename) {
         s_log(LOG_INFO, "Cannot retrieve any random data from %s",
             filename);
     /* write new random data for future seeding if it's a regular file */
-    if(global->option.rand_write && (sb.st_mode & S_IFREG)) {
+    if(global->option.rand_write && S_ISREG(sb.st_mode)) {
         writebytes=RAND_write_file(filename);
         if(writebytes==-1)
             s_log(LOG_WARNING, "Failed to write strong random data to %s - "
